#!/bin/bash
#
# Description:
#   This script allows to do MIMOSA simulations for multiple isentropic levels 
# between 2 dates. This script will also create the list of MIMOSA outputted files
# needed to use the IDL plotting program
#
# Categories:
#   MIMOSA tools
#
# Params:
#   none
# 
# Example:
#   To use this script you need to :
#      - copy this script in the directory containing MIMOSA executable (mimosa.x)
#      - make this script executable (chmod 777 script_mimosa.bash)
#      - Compile MIMOSA using the MakeFile
#      - configure the simulations in the parameters part below. If you need help 
#      you can refer to the documentation contained in the documentation directory.
#      - execute the script using ./mimosa.sh
# 
# History:
#   200903 MAD/LPC2E Creation
#   201006 MAD/LPC2E Add creation of text files needed by IDL plot program
#   201109 MAD/LPC2E Group all parameters
#                    Change creation of input.namelist due to change in MIMOSA model
#                    Add creation and checking of directory needed for the simulation
#
# Author:
#   M.-A. Drouin
#

#----------------------------------------------------------------------
# Parameters start
#----------------------------------------------------------------------

## RUN
ZONE=3
INTYPE=1
# Start date
SYEAR=11
SMONTH=4
SDAY=10
SHOUR=0
# End date
EYEAR=11
EMONTH=4
EDAY=20
EHOUR=12
INITPV=1
declare -a THETA=( 850 )

## GRID
NX=320
NY=161
NP=37
PRES="1000.,975.,950.,925.,900.,875.,850.,825.,800.,775.,750.,700.,650.,600.,550.,500.,450.,400.,350.,300.,250.,225.,200.,175.,150.,125.,100.,70.,50.,30.,20.,10.,7.,5.,3.,2.,1."
PASLAT=1.125
PASLONG=1.125
LATMIN=-90
LATMAX=90

## CONFIG
NDEG=3
NLIS2D=8
NHGRID=6
NWRITE=6
NHMOD=12
NHRELAX=240
NPRHMOD=0
NPRWRITE=0
INDIFEXPL=0
DIFF=4050

#OUTPUT
NRUN=0
NWTEMP=12
WINDOUT=1
TOUT=1
STATIONS=0

#----------------------------------------------------------------------
# Parameters end
#----------------------------------------------------------------------

echo ""
echo "Start of MIMOSA script"
echo ""

# Current directory
CURDIR=$(echo `pwd`)

#----------------------------------------------------------------------
# Check before execution
#----------------------------------------------------------------------

# Current directory
CURDIR=$(echo `pwd`)

if (( NRUN < 10 )) 
then
	RUNDIR=$(echo "RUN0"$NRUN)
else
	RUNDIR=$(echo "RUN"$NRUN)
fi

# Check if the directories needed are present if not 
# we quit or we create the directories

# Input directory
if (( INTYPE == 1 ))
then
	if [ ! -d "ECMR" ]; then
		echo ""
		echo "ERROR: can't find ECMR directory"
		echo "End of program"
		exit 1
	fi
elif (( INTYPE == 2  ))
then
	if [ ! -d "GRIB" ]; then
		echo ""
		echo "ERROR: can't find GRIB directory"
		echo "End of program"
		exit 1
	fi
fi

# Output directory
if [ ! -d $RUNDIR ]; then
	echo ""
	echo "Can't find the "$RUNDIR" directory. It will be created."
fi

#----------------------------------------------------------------------
# Executing MIMOSA simulation for each isentropic level in $THETA
#----------------------------------------------------------------------

# Loop over all the isentropic levels requested
for i in ${THETA[*]}
do

echo "run for theta = "$i"K"

# Generating the input.namelist file for each isentropic level requested
cat <<EOF >input.namelist
!======================================================================!
!               __  __ _____ __  __  ____   _____                      !
!              |  \/  |_   _|  \/  |/ __ \ / ____|  /\                 !
!              | \  / | | | | \  / | |  | | (___   /  \                !
!              | |\/| | | | | |\/| | |  | |\___ \ / /\ \               !
!              | |  | |_| |_| |  | | |__| |____) / ____ \              !
!              |_|  |_|_____|_|  |_|\____/|_____/_/    \_\             !
!======================================================================!
!
! CARACTERISCS OF THE RUN
!
&run
!-----
! ZONE defines the geographical area :
!   -> 1 for the Northern Hemisphere [-10N, 90N]
!   -> 2 for the Southern hemisphere [-90N, 10N]
!   -> 3 for the both Hemisphere     [-90N, 90N]
zone = $ZONE
!-----
! INTYPE defines the type of input files :
!   -> 1 for ASCII isobaric files (*.ECMR) 
!   -> 2 for GRIB encoded model levels files (*.grib)
intype  = $INTYPE
!-----
! IAND, MOISD, JOURD, IHEURED define the starting date of the simulation
!  -> Year  (YY)
iand    = $SYEAR
!  -> Month (MM)
moisd   = $SMONTH
!  -> Day   (DD)
jourd   = $SDAY
!  -> Hour  (HH)
iheured = $SHOUR
!-----
! IANF, MOISF, JOURF, IHEUREF define the final date of the simulation
!  -> Year  (YY) 
ianf    = $EYEAR
!  -> Month (MM)
moisf   = $EMONTH
!  -> Day   (DD)
jourf   = $EDAY
!  -> Hour  (HH)
iheuref = $EHOUR
!-----
! INITPV defines if the simulation is new or a restart
!  -> 1 if initialization is need 
!  -> 0 if a ph* file from a previous run should be read
initpv  = $INITPV
!-----
! TETA defines the isentropic surface (K)
!   -> shouldn't be greater than 950K for isobaric input files (intype = 1)
teta    = $i
/
!
! CARACTERISCS OF ECMWF GRID
!
&grid
!-----
! NX, NY and NP define the number of points of the ECMWF grid
!   -> Number of grid points along longitudes 
nx = $NX
!   -> Number of grid points along latitudes
ny = $NY
!   -> Number of pressure levels (intype = 1) or number or model levels (inType = 2) 
np = $NP
!-----
! PRES allows to define the pressure levels of isobaric file (intype = 1)
!   -> this variable is not needed for GRIB files (intype = 2)
!   -> If there is more than 50 levels, declaration of pres variable in mimosa.f95 
pres(1) = $PRES
!-----
! PASLAT and PASLONG define the horizontal resolution of the ECMWF grid
!   -> resolution along latitude
paslat = $PASLAT
!   -> resolution along longitude
paslong = $PASLONG
!-----
! LATMINECMR and LATMAXECMR define the minumum and maximum latitude of ECMWF grid
!   -> minimum latitude 
latminecmr = $LATMIN
!   -> maximum latitude
latmaxecmr = $LATMAX
/
!
! CONFIGURATION OF THE SIMULATION
!
&config
!-----
!  NDEG defines the number of MIMOSA grid points per degree of latitude and longitude
!    -> value should be 3 or 6
ndeg   = $NDEG
!-----
!  NLIS2D defines the number of points use for the smooth
nlis2d = $NLIS2D
!-----
! NHGRID defines the number of hours between two call to regrid
nhgrid = $NHGRID
!-----
! NWRITE defines the number of hour between two outputs of PV files
nwrite = $NWRITE
!-----
! NHMOD defines the number of hours between two ECMWF files
nhmod = $NHMOD
!-----
! NHRELAX defines the number of hours of relaxation time
nhrelax = $NHRELAX
!-----
! NPRHMOD defines the hour of the first ECMWF file
nprhmod = $NPRHMOD
!-----
! NPRWRITE defines the first hour of PVF file
nprwrite = $NPRWRITE
!-----
! Explicit diffusion
! INDIFEXPL defines if explicit diffusion is activated
!   -> 0 no explicit diffusion
!   -> 1 explicit diffusion
indifexpl = $INDIFEXPL
! DIFF defines the value of the explicit diffusion if it is activated
diff = $DIFF
/
!
! OUTPUT OF THE SIMULATION
!
&output
!-----
! NRUN defines the name of the directory where MIMOSA outputs will be saved
!   -> if nrun =  5, files will be saved in RUN05 directory
!   -> if nrun = 16, files will be saved in RUN16 directory
nrun = $NRUN
!-----
! NWTEMP defines the time between two output of temperature or wind     
nwtemp = $NWTEMP
!-----
! WIND_OUT defines if wind horizontal components files will be saved
!   -> 0 no output of wind files
!   -> 1 output of wind files
wind_out = $WINDOUT
!-----
! T_OUT defines if temperature files will be saved
!   -> 0 no output of temperature files
!   -> 1 output of temperature files
t_out = $TOUT
!-----
! STATIONS_OUT defines if PV and temperature and PV profiles at stations files will be saved
!   -> 0 no output of stations files
!   -> 1 output of stations files
stations_out = $STATIONS
/ 
EOF

./mimosa.x

done

# Creation of the list of files
if (( NRUN < 10 )) 
then
RUNDIR=$(echo "RUN0"$NRUN)
else
RUNDIR=$(echo "RUN"$NRUN)
fi

echo ""
echo "Creation of list files needed by IDL program"

cd $RUNDIR
for i in ${THETA[*]}
do
	echo "   - list of PV files"
	# list of potential vorticity files per theta level
	ls pv*$i > $CURDIR"/"$RUNDIR"/filelist"
	wc -l < $CURDIR"/"$RUNDIR"/filelist" > $CURDIR"/list_mimosa_pv_"$i".txt"
	rm $CURDIR"/"$RUNDIR"/filelist"
	OUTPUTLIST=`ls pv*$i`
	for j in $OUTPUTLIST
	do
		echo $j >> $CURDIR"/list_mimosa_pv_"$i".txt"
	done

	# list of temperature files per theta level
	if (( TOUT == 1 ))
	then

		echo "   - list of temperature files"
		ls t*$i > $CURDIR"/"$RUNDIR"/filelist"
		wc -l < $CURDIR"/"$RUNDIR"/filelist" > $CURDIR"/list_mimosa_t_"$i".txt"
		rm $CURDIR"/"$RUNDIR"/filelist"
		OUTPUTLIST=`ls t*$i`
		for j in $OUTPUTLIST
		do
			echo $j >> $CURDIR"/list_mimosa_t_"$i".txt"
		done

	fi

	# list of zonal wind files per theta level
	if (( WINDOUT == 1 ))
	then

		echo "   - list of wind files"
		ls u*$i > $CURDIR"/"$RUNDIR"/filelist"
		wc -l < $CURDIR"/"$RUNDIR"/filelist" > $CURDIR"/list_mimosa_u_"$i".txt"
		rm $CURDIR"/"$RUNDIR"/filelist"
		OUTPUTLIST=`ls u*$i`
		for j in $OUTPUTLIST
		do
			echo $j >> $CURDIR"/list_mimosa_u_"$i".txt"
		done

	fi

done

cd $CURDIR

echo " "
echo "END"


exit 0




