#!/bin/bash
# purpose:
#   This script is made to retrieve ECMWF fields in ECMR format 
#  needed to initialize MIMOSA model
#
# history:
#   201109 MAD/LPC2E creation
#
#
#@ shell           = /usr/bin/ksh
#@ class           = express
#@ job_name        = Mimosa_ECMR
#@ output          = /scratch/ms/fr/aut/$(job_name).$(host).$(jobid).out
#@ error           = /scratch/ms/fr/aut/$(job_name).$(host).$(jobid).out
#@ environment     = COPY_ALL
#@ notification    = error
#@ job_cpu_limit   = 00:55:00,00:54:55
#@ wall_clock_limit= 06:00:00,05:59:00
#@ queue

set -xv
export MARS_MULTITARGET_STRICT_FORMAT=1

#----------------------------------------------------------------------
# Settings
#----------------------------------------------------------------------

# Caracteristics of ECMWF T, U and V fields to retrive on MARS
YEAR1=2011			# Start year
MONTH1=5			# Start month
DAY1=31				# Start Day
YEAR2=2011			# End Year
MONTH2=6			# End month
DAY2=1				# End day
# Pressure levels in increasing order for ERAI (37 pressure levels)
M_LEVELS=1/2/3/5/7/10/20/30/50/70/100/125/150/175/200/225/250/300/350/400/450/500/550/600/650/700/750/775/800/825/850/875/900/925/950/975/1000
# Pressure levels in increasing order for ERA40 (23 pressure levels)
#_LEVELS=1/2/3/5/7/10/20/30/50/70/100/150/200/250/300/400/500/600/700/775/850/925/1000
# Pressure levels in increasing order for OD (25 pressure levels)
#_LEVELS=1/2/3/5/7/10/20/30/50/70/100/150/200/250/300/400/500/700/850/925/1000
M_STEP=6	        # Hour between 2 files ( only accept 6 or 12)
M_RESOL=2.000		# lon/lat grid resolution
M_LONG_MAX=368.000	# max lon of the grid (360 - M_RESOL)
M_CLASS=ei			# Class of model (od, ei, e4)
M_STREAM=oper		# 
M_TYPE=an			# type of data (an, fc, ...)

# What to do with the files created (save on $SCRATCH and/or send them using ECTRANS)
M_ECTRANS=1			# 1 to send files using ECTRANS else 0
M_SCRATCH=1         # 1 to save the file in directory $SCRATCH/MIMOSA/YEAR else 0
# If use of ECTRANS (see ECTRANS manual)
GATEWAY=ecaccess.ecmwf.int					# ECTRANS gateway
DESTINATION=girafe_MIMOSA					# ECTRANS DESTINATION

#--------------------------------------------------------------------- 
# Setting environment variables 
#---------------------------------------------------------------------

export PATH=$PATH:.
set -xv
export MARS_MULTITARGET_STRICT_FORMAT=1

#----------------------------------------------------------------------
# Functions to convert gregorian <-> julian days
#----------------------------------------------------------------------

# This function returns the Astro-Julian Date.  The Julian 
# date (JD) is a continuous count of days from 1 January 4713 BC. 
# The following algorithm is good from years 1801 to 2099.
# See URL: 
# http://aa.usno.navy.mil/faq/docs/JD_Formula.html 
# for more information
# arguments: $1 = day, $2 = month, $3 = year in format YYYY
get_astro_JD()
{
declare -i JDD

JDD=$(($1-32075+1461*($3+4800+($2-14)/12)/4+367*($2-2-($2-14)/12*12)/12-3*(($3+4900+($2-14)/12)/100)/4))
echo $JDD
}


# This function gives returns the gregorian YEAR from a Julian date
# The following algorithm is good from years 1801 to 2099.
# See URL: 
# http://aa.usno.navy.mil/faq/docs/JD_Formula.html 
# for more information
# argument : $1 = julian date (e.g. from get_astro_JD() function)
get_astro_YEAR()
{
declare -i MONTH
declare -i YEAR
declare -i DAY
declare -i varL
declare -i varN

      varL=$(($1+68569))
      varN=$((4*$varL/146097))
      varL=$(($varL-(146097*$varN+3)/4))
      YEAR=$((4000*($varL+1)/1461001))
      varL=$(($varL-1461*$YEAR/4+31))
      MONTH=$((80*$varL/2447))
      DAY=$(($varL-2447*$MONTH/80))
      varL=$(($MONTH/11))
      MONTH=$(($MONTH+2-12*$varL))
      YEAR=$((100*($varN-49)+$YEAR+$varL))
     
echo $YEAR
}



# This function gives returns the gregorian MONTH from a Julian date
# The following algorithm is good from years 1801 to 2099.
# See URL: 
# http://aa.usno.navy.mil/faq/docs/JD_Formula.html 
# for more information
# argument : $1 = julian date (e.g. from get_astro_JD() function)
get_astro_MONTH()
{
declare -i MONTH
declare -i YEAR
declare -i DAY
declare -i varL
declare -i varN

      varL=$(($1+68569))
      varN=$((4*$varL/146097))
      varL=$(($varL-(146097*$varN+3)/4))
      YEAR=$((4000*($varL+1)/1461001))
      varL=$(($varL-1461*$YEAR/4+31))
      MONTH=$((80*$varL/2447))
      DAY=$(($varL-2447*$MONTH/80))
      varL=$(($MONTH/11))
      MONTH=$(($MONTH+2-12*$varL))
      YEAR=$((100*($varN-49)+$YEAR+$varL))
     
echo $MONTH
}

# This function gives returns the DAY OF THE MONTH of a gregorian calendar date from a Julian date
# The following algorithm is good from years 1801 to 2099.
# See URL: 
# http://aa.usno.navy.mil/faq/docs/JD_Formula.html 
# for more information
# argument : $1 = julian date (e.g. from get_astro_JD() function)
get_astro_DAY()
{
declare -i MONTH
declare -i YEAR
declare -i DAY
declare -i varL
declare -i varN

      varL=$(($1+68569))
      varN=$((4*$varL/146097))
      varL=$(($varL-(146097*$varN+3)/4))
      YEAR=$((4000*($varL+1)/1461001))
      varL=$(($varL-1461*$YEAR/4+31))
      MONTH=$((80*$varL/2447))
      DAY=$(($varL-2447*$MONTH/80))
      varL=$(($MONTH/11))
      MONTH=$(($MONTH+2-12*$varL))
      YEAR=$((100*($varN-49)+$YEAR+$varL))
     
echo $DAY
}

#----------------------------------------------------------------------
# commands to be executed nothing need to be modified below
#----------------------------------------------------------------------

# Check the existence of require directory for M_SCRATCH=1
if [[ $M_SCRATCH == 1 ]]
then

	out=/scratch/ms/fr/${USER}/MIMOSA
	if [[ ! -d  $out ]]
	then
		mkdir $out
	fi


	out=/scratch/ms/fr/${USER}/MIMOSA/${YEAR1:0:4}
	if [[ ! -d $out ]]
	then
		mkdir $out
	fi
	
fi

TMPDIR=${SCRATCH}/scratchdir/MIMOSA_${HOST}_$$
mkdir $TMPDIR
cd $TMPDIR

# FORTRAN program to convert GRIB fields to ECMR format
cat>MimosaECMR.f90<<EOF
! Program:
!    mimosaECMR
!
! Purpose:
!    This program is made to convert ECMWF fields in GRIB1/2 format
!   to the ECMR format used by MIMOSA.
!
! Input:
!    none
!
! Output:
!    none
!
! Restrictions:
!    none
!
! Comments:
!    This program is only made to deal we GRIB files containing 3 parameters
!   Nevertheless you just have to change the parameter NPARAMS to make it usable for more parameters
!
! History
!    201109   M.-A. DROUIN/LPC2E creation
!
!======================================================================
program mimosaECMR

    use grib_api

    implicit none

    !==================================================================
    ! Declarations & Initialisation
    !==================================================================

    ! Control of grib file
    !------------------------------------------------------------------
    integer                            :: iFile
    integer, dimension(:), allocatable :: iGrib
    integer                            :: nMess
    integer                            :: iRet

    ! caracteristics of the data
    !-------------------------------------------------------------------
    integer, parameter :: nParams = 3
    integer            :: day
    integer            :: month
    integer            :: lyear
    integer            :: syear
    integer            :: hour
    integer            :: nLevels
    integer            :: nx
    integer            :: ny
    integer            :: ndata

    real                            :: level
    real                            :: alatdeb
    real                            :: alondeb
    real                            :: pasx
    real                            :: pasy
    real, dimension(:), allocatable :: tab

    character(len=25) :: name
    character(len=25) :: units
    character(len=14) :: nomfic

    ! Loop counters
    !------------------------------------------------------------------
    integer :: i
    integer :: j
    integer :: k

    ! Format
    !------------------------------------------------------------------
    character(len=19),  parameter :: FMT100 = "('D',4I2.2,'.ECMR')"
    character(len=155), parameter :: FMT101 = "(1x,'Echeance',i5,3i3&
                                              &, /,1x,'Parameter',1x,a25&
                                              &, /,1x,'Unit ',a25&
                                              &, /,1x,'Level ',f6.1,' hPa'&
                                              &, /,1x,'grid longitude',i4,2f8.3,/,1x,'grid latitude ',i4,2f8.3)"
    character(len=8),   parameter :: FMT102 = "(10f7.2)"


    !======================================================================
    ! Program
    !======================================================================

    ! Opening grib file
    call grib_open_file(iFile, 'datafile', 'r', iRet)
    if (iret .ne. GRIB_SUCCESS) then
        print *, 'ERROR : file cannot be opened'
    endif

    ! Count the message in the file
    call grib_count_in_file(iFile, nMess)
    allocate(igrib(nMess))
    iGrib = -1

    ! number of pressure levels
    nLevels = nMess/nParams
    print *, 'the file contains ', nLevels, 'pressure levels'

    ! Load the message from the file
    do i = 1, nMess
        call grib_new_from_file(iFile,iGrib(i),iRet)
    end do

    ! We can close the file
    call grib_close_file(iFile)

    !  We loop over all the message knowing that
    ! messages are organize like this:
    !  - LEVEL 1 PARAM 1
    !  - LEVEL 1 PARAM 2
    !  - LEVEL 1 PARAM 3
    ! ....
    !  - LEVEL m PARAM 1
    !  - LEVEL m PARAM 3
    !  - LEVEL m PARAM 3
    params: do i = 1, 3

        levels: do j = i, nMess, nParams

            ! Get all necessary info for level header
            call grib_get_int(iGrib(j), 'day', day, iRet)
            call grib_get_int(iGrib(j), 'month', month, iRet)
            call grib_get_int(iGrib(j), 'year', lyear, iRet)
            call grib_get_int(iGrib(j), 'hour', hour, iRet)
            call grib_get_real4(iGrib(j), 'longitudeOfFirstGridPointInDegrees', alondeb, iRet)
            call grib_get_real4(iGrib(j), 'latitudeOfLastGridPointInDegrees', alatdeb, iRet)
            call grib_get_real4(iGrib(j), 'iDirectionIncrementInDegrees', pasx, iRet)
            call grib_get_real4(iGrib(j), 'jDirectionIncrementInDegrees', pasy, iRet)
            call grib_get_int(iGrib(j), 'Ni', nx, iRet)
            call grib_get_int(iGrib(j), 'Nj', ny, iRet)
            call grib_get_string(iGrib(j), 'name', name, iRet)
            call grib_get_string(iGrib(j), 'units', units, iRet)

            if (lyear < 2000 ) then
                syear = lyear-1900
            else
                syear = lyear-2000
            endif

            if (j == 1 .and. i == 1) then
                write(nomfic,FMT100) syear, month, day, hour
                open(11,file=nomfic)
            end if

            call grib_get_real4(iGrib(j), 'level', level, iRet)
            call grib_get_size(iGrib(j), 'values', ndata, iRet)
            allocate(tab(ndata))
            call grib_get_real4_array(iGrib(j), 'values', tab, iRet)

            write(11,FMT101) lyear, month, day, hour, name, units, level, nx, alondeb, pasx, ny, alatdeb, pasy
            do k=1,ny
                write(11,FMT102) tab(((ny-k)*nx)+1:(ny-k)*nx+nx)
            end do
            deallocate(tab)

        end do levels

    end do params
    
    close(11)

    do i = 1, nMess
        call grib_release(iGrib(i))
    end do

    deallocate(iGrib)

end program mimosaECMR
EOF

# Compilation of the FORTRAN program
xlf90 -qsuffix=f=f90 -o MimosaECMR MimosaECMR.f90 $GRIB_API_INCLUDE $GRIB_API_LIB

# Convert gregorian day to julian day
JUL1=$(get_astro_JD $((10#$DAY1)) $((10#$MONTH1)) $YEAR1)
JUL2=$(get_astro_JD $((10#$DAY2)) $((10#$MONTH2)) $YEAR2)

# Check the date
if (( JUL1 > JUL2 ))
then
	
	echo "ERROR: start date should be smaller than end date."
	echo "check the value of YEAR1, MONTH1, DAY1, YEAR2, MONTH2 and DAY2."
	exit 1
fi

while (( JUL1 <= JUL2 ))
do

	YEAR=$(get_astro_YEAR $JUL1)
	MONTH=$(get_astro_MONTH $JUL1)
	DAY=$(get_astro_DAY $JUL1)
	echo $YEAR $MONTH $DAY
	if (( MONTH < 10 )) 
		then MONTH="0"$MONTH
	fi
	if (( DAY < 10 )) 
		then DAY="0"$DAY
	fi

	DATE1=$YEAR$MONTH$DAY
	echo $DATE1

cat >request_$$ <<EOF
retrieve,
    class    = $M_CLASS,
    stream   = $M_STREAM,
    date     = $DATE1,
    type     = $M_TYPE,
    time     = 00,
    step     = 00,
    levtype  = pl,
    levels   = ${M_LEVELS},
    area     = 90./0./-90./$M_LONG_MAX,
    resol    = 106,
    grid     = $M_RESOL/$M_RESOL,
    param    = 130/131/132,
    target   = "G.[date][time]"
	
retrieve,
    class    = $M_CLASS,
    stream   = $M_STREAM,
    date     = $DATE1,
    type     = $M_TYPE,
    time     = 12,
    step     = 00,
    levtype  = pl,
    levels   = ${M_LEVELS},
    area     = 90./0./-90./$M_LONG_MAX,
    resol    = 106,
    grid     = $M_RESOL/$M_RESOL,
    param    = 130/131/132,
    target   = "G.[date][time]"

EOF

	mars request_$$

	# Check MARS exit code.
	if [[ $? != 0 ]]
	then
      	echo " The MARS request failed for date "$DATE1
		exit 1
	fi

	if [[ $M_STEP == 6 ]]
	then

cat >request_$$ <<EOF
retrieve,
    class    = $M_CLASS,
    stream   = $M_STREAM,
    date     = $DATE1,
    type     = $M_TYPE,
    time     = 06,
    step     = 00,
    levtype  = pl,
    levels   = ${M_LEVELS},
    area     = 90./0./-90./$M_LONG_MAX,
    resol    = 106,
    grid     = $M_RESOL/$M_RESOL,
    param    = 130/131/132,
    target   = "G.[date][time]"
	
retrieve,
    class    = $M_CLASS,
    stream   = $M_STREAM,
    date     = $DATE1,
    type     = $M_TYPE,
    time     = 18,
    step     = 00,
    levtype  = pl,
    levels   = ${M_LEVELS},
    area     = 90./0./-90./$M_LONG_MAX,
    resol    = 106,
    grid     = $M_RESOL/$M_RESOL,
    param    = 130/131/132,
    target   = "G.[date][time]"

EOF

	mars request_$$

	fi

	# Check MARS exit code.
	if [[ $? != 0 ]]
	then
      	echo " The MARS request failed for date "$DATE1
		exit 1
	fi

	# Naming files with the right format
	INFILE=`ls G*`
	for i in $INFILE
	do
		ln -s $i datafile
		./MimosaECMR
		rm datafile
	done
	
	# Transfer and/or save them
	INFILE=`ls D*`
	for i in $INFILE
	do
		if [[ $M_ECTRANS == 1 ]]
		then
			ectrans -overwrite -gateway ${GATEWAY} -remote ${DESTINATION} -source ${i}
			if [[ $M_SCRATCH != 1 ]]
			then
				rm ${i}
			fi
		fi	

		if [[ $M_SCRATCH == 1 ]]
		then
			mv $i $out
		fi
		
	done
	
	rm G*
	
	# Increment of JUL1
	(( JUL1++ ))

done

exit 0
