#!/bin/bash
# purpose:
#   This script is made to retrieve ECMWF fields in GRIB format 
#  needed to initialize MIMOSA model
#
# comments:
#   This script is compatible with ECMWF data using GRIB2 format
#
# history:
#   201109 MAD/LPC2E creation
#
#@ shell           = /usr/bin/ksh
#@ class           = express
#@ job_name        = Mimosa_grib
#@ output          = /scratch/ms/fr/aut/$(job_name).$(host).$(jobid).out
#@ error           = /scratch/ms/fr/aut/$(job_name).$(host).$(jobid).out
#@ environment     = COPY_ALL
#@ notification    = error
#@ job_cpu_limit   = 00:55:00,00:54:55
#@ wall_clock_limit= 06:00:00,05:59:00
#@ queue

set -xv
export MARS_MULTITARGET_STRICT_FORMAT=1

#----------------------------------------------------------------------
# Settings
#----------------------------------------------------------------------

# Caracteristics of ECMWF lnsp, T, U and V fields to retrieve on MARS
YEAR1=2011			# Start year
MONTH1=5			# Start month
DAY1=31				# Start Day
YEAR2=2011			# End Year
MONTH2=6			# End month
DAY2=1				# End day
M_LEVELS=91			# Number of vertical levels
M_STEP=6	        # Hour between 2 files ( only accept 6 or 12)
M_RESOL=2.000		# lon/lat grid resolution
M_LONG_MAX=358.000	# max lon of the grid (360 - M_RESOL)
M_CLASS=od			# Class of model (od, ei, e4)
M_STREAM=oper		# 
M_TYPE=an			# type of data (an, fc, ...)

# What to do with the files created (save on $SCRATCH and/or send them using ECTRANS)
M_ECTRANS=1			# 1 to send files using ECTRANS else 0
M_SCRATCH=1         # 1 to save the file in directory $SCRATCH/MIMOSA/YEAR else 0
# If use of ECTRANS (see ECTRANS manual)
GATEWAY=ecaccess.ecmwf.int					# ECTRANS gateway
DESTINATION=girafe_MIMOSA					# ECTRANS DESTINATION

#--------------------------------------------------------------------- 
# Setting environment variables 
#---------------------------------------------------------------------

export PATH=$PATH:.
set -xv
export MARS_MULTITARGET_STRICT_FORMAT=1

#----------------------------------------------------------------------
# Functions to convert gregorian <-> julian days
#----------------------------------------------------------------------

# This function returns the Astro-Julian Date.  The Julian 
# date (JD) is a continuous count of days from 1 January 4713 BC. 
# The following algorithm is good from years 1801 to 2099.
# See URL: 
# http://aa.usno.navy.mil/faq/docs/JD_Formula.html 
# for more information
# arguments: $1 = day, $2 = month, $3 = year in format YYYY
get_astro_JD()
{
declare -i JDD

JDD=$(($1-32075+1461*($3+4800+($2-14)/12)/4+367*($2-2-($2-14)/12*12)/12-3*(($3+4900+($2-14)/12)/100)/4))
echo $JDD
}


# This function gives returns the gregorian YEAR from a Julian date
# The following algorithm is good from years 1801 to 2099.
# See URL: 
# http://aa.usno.navy.mil/faq/docs/JD_Formula.html 
# for more information
# argument : $1 = julian date (e.g. from get_astro_JD() function)
get_astro_YEAR()
{
declare -i MONTH
declare -i YEAR
declare -i DAY
declare -i varL
declare -i varN

      varL=$(($1+68569))
      varN=$((4*$varL/146097))
      varL=$(($varL-(146097*$varN+3)/4))
      YEAR=$((4000*($varL+1)/1461001))
      varL=$(($varL-1461*$YEAR/4+31))
      MONTH=$((80*$varL/2447))
      DAY=$(($varL-2447*$MONTH/80))
      varL=$(($MONTH/11))
      MONTH=$(($MONTH+2-12*$varL))
      YEAR=$((100*($varN-49)+$YEAR+$varL))
     
echo $YEAR
}



# This function gives returns the gregorian MONTH from a Julian date
# The following algorithm is good from years 1801 to 2099.
# See URL: 
# http://aa.usno.navy.mil/faq/docs/JD_Formula.html 
# for more information
# argument : $1 = julian date (e.g. from get_astro_JD() function)
get_astro_MONTH()
{
declare -i MONTH
declare -i YEAR
declare -i DAY
declare -i varL
declare -i varN

      varL=$(($1+68569))
      varN=$((4*$varL/146097))
      varL=$(($varL-(146097*$varN+3)/4))
      YEAR=$((4000*($varL+1)/1461001))
      varL=$(($varL-1461*$YEAR/4+31))
      MONTH=$((80*$varL/2447))
      DAY=$(($varL-2447*$MONTH/80))
      varL=$(($MONTH/11))
      MONTH=$(($MONTH+2-12*$varL))
      YEAR=$((100*($varN-49)+$YEAR+$varL))
     
echo $MONTH
}

# This function gives returns the DAY OF THE MONTH of a gregorian calendar date from a Julian date
# The following algorithm is good from years 1801 to 2099.
# See URL: 
# http://aa.usno.navy.mil/faq/docs/JD_Formula.html 
# for more information
# argument : $1 = julian date (e.g. from get_astro_JD() function)
get_astro_DAY()
{
declare -i MONTH
declare -i YEAR
declare -i DAY
declare -i varL
declare -i varN

      varL=$(($1+68569))
      varN=$((4*$varL/146097))
      varL=$(($varL-(146097*$varN+3)/4))
      YEAR=$((4000*($varL+1)/1461001))
      varL=$(($varL-1461*$YEAR/4+31))
      MONTH=$((80*$varL/2447))
      DAY=$(($varL-2447*$MONTH/80))
      varL=$(($MONTH/11))
      MONTH=$(($MONTH+2-12*$varL))
      YEAR=$((100*($varN-49)+$YEAR+$varL))
     
echo $DAY
}


#----------------------------------------------------------------------
# commands to be executed nothing need to be modified below
#----------------------------------------------------------------------

# Check the existence of require directory for M_SCRATCH=1
if [[ $M_SCRATCH == 1 ]]
then

	out=/scratch/ms/fr/${USER}/MIMOSA
	if [[ ! -d  $out ]]
	then
		mkdir $out
	fi


	out=/scratch/ms/fr/${USER}/MIMOSA/${YEAR1:0:4}
	if [[ ! -d $out ]]
	then
		mkdir $out
	fi
	
fi

TMPDIR=${SCRATCH}/scratchdir/MIMOSA_${HOST}_$$
mkdir $TMPDIR
cd $TMPDIR

# Convert gregorian day to julian day
# For calculation we make sure $DAY and $MONTH are considered as integer in base 10
JUL1=$(get_astro_JD $((10#$DAY1)) $((10#$MONTH1)) $YEAR1)
JUL2=$(get_astro_JD $((10#$DAY2)) $((10#$MONTH2)) $YEAR2)

# Check the date
if (( JUL1 > JUL2 ))
then
	
	echo "ERROR: start date should be smaller than end date."
	echo "check the value of YEAR1, MONTH1, DAY1, YEAR2, MONTH2 and DAY2."
	exit 1
fi

while (( JUL1 <= JUL2 ))
do

	YEAR=$(get_astro_YEAR $JUL1)
	MONTH=$(get_astro_MONTH $JUL1)
	DAY=$(get_astro_DAY $JUL1)
	echo $YEAR $MONTH $DAY
	if (( MONTH < 10 )) 
		then MONTH="0"$MONTH
	fi
	if (( DAY < 10 )) 
		then DAY="0"$DAY
	fi

	DATE1=$YEAR$MONTH$DAY
	echo $DATE1

cat >request_$$ <<EOF
retrieve,
    class    = $M_CLASS,
    stream   = $M_STREAM,
    date     = $DATE1,
    type     = $M_TYPE,
    time     = 00,
    step     = 00,
    levtype  = ml,
    levels   = 1,
    area     = 90./0./-90./$M_LONG_MAX,
    resol    = 106,
    grid     = $M_RESOL/$M_RESOL,
    param    = lnsp,
    target   = "D[date][time].grib"
retrieve,
    levels   = 1/to/$M_LEVELS,
    param    = 130/131/132,
    target   = "D[date][time].grib"
	
retrieve,
    class    = $M_CLASS,
    stream   = $M_STREAM,
    date     = $DATE1,
    type     = $M_TYPE,
    time     = 12,
    step     = 00,
    levtype  = ml,
    levels   = 1,
    area     = 90./0./-90./$M_LONG_MAX,
    resol    = 106,
    grid     = $M_RESOL/$M_RESOL,
    param    = lnsp,
    target   = "D[date][time].grib"
retrieve,
    levels   = 1/to/$M_LEVELS,
    param    = 130/131/132,
    target   = "D[date][time].grib"

EOF

	mars request_$$

	# Check MARS exit code.
	if [[ $? != 0 ]]
	then
      	echo " The MARS request failed for date "$DATE1
		exit 1
	fi

	if [[ $M_STEP == 6 ]]
	then

cat >request_$$ <<EOF
retrieve,
    class    = $M_CLASS,
    stream   = $M_STREAM,
    date     = $DATE1,
    type     = $M_TYPE,
    time     = 06,
    step     = 00,
    levtype  = ml,
    levels   = 1,
    area     = 90./0./-90./$M_LONG_MAX,
    resol    = 106,
    grid     = $M_RESOL/$M_RESOL,
    param    = lnsp,
    target   = "D[date][time].grib"
retrieve,
    levels   = 1/to/$M_LEVELS,
    param    = 130/131/132,
    target   = "D[date][time].grib"
	
retrieve,
    class    = $M_CLASS,
    stream   = $M_STREAM,
    date     = $DATE1,
    type     = $M_TYPE,
    time     = 18,
    step     = 00,
    levtype  = ml,
    levels   = 1,
    area     = 90./0./-90./$M_LONG_MAX,
    resol    = 106,
    grid     = $M_RESOL/$M_RESOL,
    param    = lnsp,
    target   = "D[date][time].grib"
retrieve,
    levels   = 1/to/$M_LEVELS,
    param    = 130/131/132,
    target   = "D[date][time].grib"

EOF

	mars request_$$

	fi

	# Check MARS exit code.
	if [[ $? != 0 ]]
	then
      	echo " The MARS request failed for date "$DATE1
		exit 1
	fi

	# Naming files with the right format and transfer and/or save them
	INFILE=`ls D*`
	for i in $INFILE
	do
		year=${i:1:4}
		tmp1=${i:3:8}
		tmp2=${i:13:5}
		fileout=D${tmp1}${tmp2}
		mv $i $fileout
											
		if [[ $M_ECTRANS == 1 ]]
		then
			ectrans -overwrite -gateway ${GATEWAY} -remote ${DESTINATION} -source ${fileout}
			if [[ $M_SCRATCH != 1 ]]
			then
				rm ${fileout}
			fi
		fi	

		if [[ $M_SCRATCH == 1 ]]
		then
		mv $fileout $out
		fi
		
	done
	
	# Increment of JUL1
	(( JUL1++ ))

done

exit 0
