! Program:
!    mimosaECMR
!
! Purpose:
!    This program is made to convert ECMWF fields in GRIB1/2 format
!   to the ECMR format used by MIMOSA.
!    The program is made to open a GRIB formatted file named "datafile"
!   which it will convert to ECMR format
!
! Input:
!    none
!
! Output:
!    none
!
! Restrictions:
!    none
!
! Comments:
!    This program is only made to deal we GRIB files containing 3 parameters
!   Nevertheless you just have to change the parameter NPARAMS to make it usable for more parameters
!
! History
!    201109   M.-A. DROUIN/LPC2E creation
!
!======================================================================
program mimosaECMR

    use grib_api

    implicit none

    !==================================================================
    ! Declarations & Initialisation
    !==================================================================

    ! Control of grib file
    !------------------------------------------------------------------
    integer                            :: iFile
    integer, dimension(:), allocatable :: iGrib
    integer                            :: nMess
    integer                            :: iRet

    ! caracteristics of the data
    !-------------------------------------------------------------------
    integer, parameter :: nParams = 3
    integer            :: day
    integer            :: month
    integer            :: lyear
    integer            :: syear
    integer            :: hour
    integer            :: nLevels
    integer            :: nx
    integer            :: ny
    integer            :: ndata

    real                            :: level
    real                            :: alatdeb
    real                            :: alondeb
    real                            :: pasx
    real                            :: pasy
    real, dimension(:), allocatable :: tab

    character(len=25) :: name
    character(len=25) :: units
    character(len=14) :: nomfic

    ! Loop counters
    !------------------------------------------------------------------
    integer :: i
    integer :: j
    integer :: k

    ! Format
    !------------------------------------------------------------------
    character(len=19),  parameter :: FMT100 = "('D',4I2.2,'.ECMR')"
    character(len=155), parameter :: FMT101 = "(1x,'Echeance',i5,3i3&
                                              &, /,1x,'Parameter',1x,a25&
                                              &, /,1x,'Unit ',a25&
                                              &, /,1x,'Level ',f6.1,' hPa'&
                                              &, /,1x,'grid longitude',i4,2f8.3,/,1x,'grid latitude ',i4,2f8.3)"
    character(len=8),   parameter :: FMT102 = "(10f7.2)"


    !======================================================================
    ! Program
    !======================================================================

    ! Opening grib file
    call grib_open_file(iFile, 'datafile', 'r', iRet)
    if (iret .ne. GRIB_SUCCESS) then
        print *, 'ERROR : file cannot be opened'
    endif

    ! Count the message in the file
    call grib_count_in_file(iFile, nMess)
    allocate(igrib(nMess))
    iGrib = -1

    ! number of pressure levels
    nLevels = nMess/nParams
    print *, 'the file contains ', nLevels, 'pressure levels'

    ! Load the message from the file
    do i = 1, nMess
        call grib_new_from_file(iFile,iGrib(i),iRet)
    end do

    ! We can close the file
    call grib_close_file(iFile)

    !  We loop over all the message knowing that
    ! messages are organize like this:
    !  - LEVEL 1 PARAM 1
    !  - LEVEL 1 PARAM 2
    !  - LEVEL 1 PARAM 3
    ! ....
    !  - LEVEL m PARAM 1
    !  - LEVEL m PARAM 3
    !  - LEVEL m PARAM 3
    params: do i = 1, 3

        levels: do j = i, nMess, nParams

            ! Get all necessary info for level header
            call grib_get_int(iGrib(j), 'day', day, iRet)
            call grib_get_int(iGrib(j), 'month', month, iRet)
            call grib_get_int(iGrib(j), 'year', lyear, iRet)
            call grib_get_int(iGrib(j), 'hour', hour, iRet)
            call grib_get_real4(iGrib(j), 'longitudeOfFirstGridPointInDegrees', alondeb, iRet)
            call grib_get_real4(iGrib(j), 'latitudeOfLastGridPointInDegrees', alatdeb, iRet)
            call grib_get_real4(iGrib(j), 'iDirectionIncrementInDegrees', pasx, iRet)
            call grib_get_real4(iGrib(j), 'jDirectionIncrementInDegrees', pasy, iRet)
            call grib_get_int(iGrib(j), 'Ni', nx, iRet)
            call grib_get_int(iGrib(j), 'Nj', ny, iRet)
            call grib_get_string(iGrib(j), 'name', name, iRet)
            call grib_get_string(iGrib(j), 'units', units, iRet)

            if (lyear < 2000 ) then
                syear = lyear-1900
            else
                syear = lyear-2000
            endif

            if (j == 1 .and. i == 1) then
                write(nomfic,FMT100) syear, month, day, hour
                open(11,file=nomfic)
            end if

            call grib_get_real4(iGrib(j), 'level', level, iRet)
            call grib_get_size(iGrib(j), 'values', ndata, iRet)
            allocate(tab(ndata))
            call grib_get_real4_array(iGrib(j), 'values', tab, iRet)

            write(11,FMT101) lyear, month, day, hour, name, units, level, nx, alondeb, pasx, ny, alatdeb, pasy
            do k=1,ny
                write(11,FMT102) tab(((ny-k)*nx)+1:(ny-k)*nx+nx)
            end do
            deallocate(tab)

        end do levels

    end do params

    close(11)

    do i = 1, nMess
        call grib_release(iGrib(i))
    end do

    deallocate(iGrib)

end program mimosaECMR
