;+
; :Description:
;    This procedure is made to plot the isosurface
;
; :Params:
;    inDir, In, Require, Type=string
;         The directory where MIMOSA output PV files are stored
;
;    outDir, In, Require, Type=string
;          The directory where the PNG 24 bit image will be store
;
;    begYear, In, Require, Type=integer
;           The year of the first date to search
;
;    begMonth, In, Require, Type=integer
;            The month of the first date to search
;
;    begDay, In, Require, Type=integer
;           The day of the first date to search
;
;    begHour, In, Require, Type=integer
;           The hour of the first date to search
;
;    endYear, In, Require, Type=integer
;           The year of the last date to search
;
;    endMonth, In, Require, Type=integer
;            The month of the last date to search
;
;    endDay, In, Require, Type=integer
;          The day of the last date to search
;
;    endHour, In, Require, Type=integer
;           The hour of the lat date to search
;
;    stepHour, In, Require, Type=integer
;            The time beteween 2 MIMOSA output files in hours
;
;    normalizeValue, In Require, Type=float
;                  The value of theta0 needed to normalize potential vorticity laitPV = PV*(theta/theta0)^(-9./2.)
;
;    contourValue, In, Require, Type=float
;                The value of the isosurface to plot
;
;    hemisphere, In, Require, Type=string
;              this parameter can take 2 values:
;               - 'N' for an azimuthal projection centered on the north pole
;               - 'S' for an azimuthal projection centered on the north pole
;
; :Keywords:
;    LAT_MIN, In, Optional, Type=float
;           for a plot on the northern hemisphere is the lowest latitude for which the isosurface will be search
;           for a plot on the southern hemisphere is the highest latitude for which the isosurface will be search
;           if not selected lat_min = 60N for hemisphere = 'N' and lat_min = -60N for hemisphere = 'S'
;
;    RESOLUTION, In, Optional, Type=intarr(2)
;              the resolution of the PNG files
;              if not selected PNG images will have a resolution of [2048, 1536]
;              If you chose a lower resolution, It could create problem with comments on the image
;
; :File_comments:
;   If the values of NORMALIZEVALUE and CONTOURVALUE doesn't allow ITTVIS procedure SHADE_VOLUME to find an isosurface in the data
;   it can induce a crash of IDL and IDL workbench
;
;   This program requires the COYOTE library. It is available at http://www.idlcoyote.com/documents/programs.php#COYOTE_LIBRARY_DOWNLOAD
;   Copyright (c) 2011, by Fanning Software Consulting, Inc. All rights reserved.
;
; :Example:
;   To plot all the surface of volume of all MIMOSA output contained in a RUN00 MIMOSA directory:
;     - between the 1st of january 2011 and the 30th of April evey 12 hours
;     - for laitPV normalized with 380K
;     - the surface of the volume with a value of 15 pvu
;     - on the northern hemisphere
;     - for latitude between 50N and 90N
;
;   IDL> MIMOSA_3D_VORTEX, '/home/madrouin/MIMOSA/RUN00/', '/home/madrouin/MIMOSA/PLOT/' $
;          , 2011,  1,  1,  0 $
;          , 2011,  4, 30, 12 $
;          , 12 $
;          , 380., 15. $
;          , 'N' $
;          , LAT_MIN = 50.
;
; :History:
;   201104 MAD/LPC2E Creation
;   201109 MAD/LPC2E add the possibility to plot volume on the south hemisphere
;                    change comments on plots
;                    use of different color palette for internal and external isosurface
;                    add a checking of the isosurface to plot
;                    use of coyote library to manage color
;                    theta in title and axis now appears as a real greek letter
;                    add comments
;
; :Author: madrouin
;-

;+
; :Description:
;    This procedure search all MIMOSA PV outputs in a directory for a given date and
;   then calculate the laitPV (Lait, 1993) and determine the vertex constituting the internal
;   and external surface of the volume
;
; :Params:
;    inDir, In, Require, Type=string
;      The directory where MIMOSA output PV files are stored
;
;    month, In, Require, Type=integer
;      The month of the files to search
;
;    day, In, Require, Type=integer
;      The day of the files to search
;
;    hour, In, Require, Type=integer
;      The hour of the files to search
;
;    year, In Require, Type=integer
;      The Year of the files to search
;
;    normalizeValue, In, Require, Type=float
;      The value use to normalize potential vorticity
;
;    contourValue, In, Require, Type=Float
;      The value of laitPV for which we search the surface
;
;    hemisphere, In, Require, Type=string
;      if hemisphere='N', the surface will be search on the northern hemisphere
;      if hemisphere='S', the surface will be search on the southern hemisphere
;
;    volume, Out, Require, Type=structure
;      The structure containing all the data necessary to plot the surface
;      volume = { laitPV   : Type=fltarr(N_ELEMENTS(lon),N_ELEMENTS(lat),N_ELEMENTS(theta)), the laitPV grid calulated using MIMOSA output
;               , lat      : Type=fltarr, the latitude vector
;               , lon      : Type=fltarr, the longitude vector
;               , VxLow    : Type=fltarr, vector containing the position along longitude of the vertex of the internal surface
;               , VyLow    : Type=fltarr, vector containing the position along latitude  of the vertex of the internal surface
;               , VzLow    : Type=fltarr, vector containing the position along theta     of the vertex of the internal surface
;               , Plow     : Type=lonarr, the polygon list of the internal surface
;               , VxHigh   : Type=fltarr, vector containing the position along longitude of the vertex of the external surface
;               , VyHigh   : Type=fltarr, vector containing the position along latitude  of the vertex of the external surface
;               , VzHigh   : Type=fltarr, vector containing the position along theta     of the vertex of the external surface
;               , PHigh    : Type=lonarr, the polygon list of the external surface
;               , theta    : Type=fltarr, the vector of theta levels read in MIMOSA files
;               , strYear  : Type=string, the year  of the data
;               , strMonth : Type=string, the month of the data
;               , strDay   : Type=string, the day   of the data
;               , strHour  : Type=string, the hour  of the data
;               , isVertexLowPresent  : Type=logical, allows to now if vertex were found for the internal surface
;               , isVertexHighPresent : Type=logical, allows to now if vertex were found for the external surface
;               }
;      If the procedure have encountered an error 0 is return in volume
;
; :Keywords:
;    LAT_MIN, In, Optional, Type=float
;      for a plot on the northern hemisphere is the lowest latitude for which the isosurface will be search
;      for a plot on the southern hemisphere is the highest latitude for which the isosurface will be search
;      if not selected lat_min = 60N for hemisphere = 'N' and lat_min = -60N for hemisphere = 'S'
;
; :History:
;    201104 MAD/LPC2E Creation
;    201109 MAD/LPC2E Add comments
;                     Add value of isVertexLowPresent and isVertexHighPresent to try to prevent some crash of IDL workbench
;                     when the value of contourValue is ill chosen. prevents some crash, not all (IDL version > 7.1 not tested for older one)
;
; :Author: madrouin
;-
pro MIMOSA_3D_laitPV_PREPARE $
    , inDir          $
    , month          $
    , day            $
    , hour           $
    , year           $
    , normalizeValue $
    , contourValue   $
    , hemisphere     $
    , volume         $
    , LAT_MIN = lat_min
    
  ; Check path
  lastInDir = STRMID(inDir, STRLEN(inDir)-1, 1)
  if ( STRCMP(lastInDir,'/') eq 0 or STRCMP(lastInDir,'\') eq 0) then inDir = inDir+PATH_SEP()
  
  ; Initialization
  error = 0
  
  ; Format of the date to create the file pattern which will be searched
  strYear      = STRING(year,  FORMAT='(I4.4)')
  shortStrYear = STRMID(strYear, 2, 2)
  strMonth     = STRING(month, FORMAT='(I2.2)')
  strDay       = STRING(day,   FORMAT='(I2.2)')
  strHour      = STRING(hour,  FORMAT='(I2.2)')
  
  pattern = 'pv?'+shortStrYear+strMonth+strDay+strHour+'.*'
  print, 'Search of pattern ' + pattern
  inFiles  = FILE_SEARCH(inDir, pattern)
  nInFiles = N_ELEMENTS(inFiles)
  
  ; Checking if we have at least two files for the same date
  if nInFiles eq 1 then begin
    error  = 1
  endif else begin
  
    ; Reading MIMOSA PV files
    MIMOSA_READ_BIN, inFiles[0], HEADER=header, DATA=data, DIMX=dimX, DIMY=dimY
    
    ;  Creating latitude and longitude vectors based on the data in the header
    ; of first read MIMOSA PV file
    lon = FINDGEN(dimX)*360./(dimX)
    lat  = header[12] - FINDGEN(dimY)*(ABS(header[12]-header[10]))/dimY
    
    ; Arrays containing theta levels and laitPV fields
    theta  = FLTARR(nInFiles)
    laitPV = FLTARR(dimX, dimY, nInFiles)
    
    theta[0]      = FLOAT(header[8])
    ; Calculus of laitPV field
    laitPV[*,*,0] = data*((theta[0]/normalizeValue)^(-9./2.))
    
    ; Reading of the others files
    for i = 1L, NinFiles-1 do begin
      MIMOSA_READ_BIN, inFiles[i], HEADER=header, DATA=data
      theta[i]      = FLOAT(header[8])
      laitPV[*,*,i] = data*((theta[i]/normalizeValue)^(-9./2.))
    endfor
    
    ;  Based on the hemisphere chosen we limit the geographical area
    ; where we search for the vortex:
    ;    -  60N to  90N for the Northern heamisphere
    ;    - -60N to -90N for the Southern hemisphere
    ;  We fill the rest of the array with !VALUES.F_NAN to prevent these area to be
    ; taken into account in the calculation of the surfaces of the volume
    case (hemisphere) of
      'N': begin
        if (N_ELEMENTS(lat_min) eq 0) then lat_min = 60.
        tmp = MIN(ABS(lat-lat_min),indLat)
        indLatMax = N_ELEMENTS(lat)-1
        laitPV[*,indLat:indLatMax,*] = !VALUES.F_NAN
      end
      'S': begin
        if (N_ELEMENTS(lat_min) eq 0) then lat_min = -60.
        tmp = MIN(ABS(lat-lat_min),indLat)
        indLatMin = 0
        laitPV[*,indLatMin:indLat,*] = !VALUES.F_NAN
      end
      else: begin
        error = 1
        PRINT, 'ERROR : hemisphere should have the value N or S'
      end
    endcase
    
    ; Calculation of internal surfaces of the volume
    SHADE_VOLUME, LaitPV, contourValue, V, PLow, /LOW
    if size(V,/TYPE) NE 0 then begin
      isVertexLowPresent = 1
      sizeV = SIZE(V, /DIMENSIONS)
      VxLow = FLTARR(sizeV[1])
      VyLow = FLTARR(sizeV[1])
      VzLow = FLTARR(sizeV[1])
      
      ;  Convertion to latitude/longitude/theta coordinates of the locations of the
      ; polygons constituting the internal surfaces
      for i = 0L, sizeV[1]-1 do begin
        VxLow[i] =  V[0,i]*360./dimX
        VyLow[i] =  FLOAT(header[12])- V[1,i]*ABS(header[12]-header[10])/dimY
        VzLow[i] =  theta[0] + V[2,i]*(theta[nInFiles-1]-theta[0])/nInFiles
      endfor
      
    endif else begin
      isVertexLowPresent = 0
    endelse
    
    ; Calculation of external surfaces of the volume
    SHADE_VOLUME, LaitPV, contourValue, V, PHigh
    if size(V,/TYPE) NE 0 then begin
      isVertexHighPresent = 1
      sizeV = SIZE(V, /DIMENSIONS)
      VxHigh = FLTARR(sizeV[1])
      VyHigh = FLTARR(sizeV[1])
      VzHigh = FLTARR(sizeV[1])
      
      ;  Convertion to latitude/longitude/theta coordinates of the locations of the
      ; polygons constituting the external surfaces
      for i = 0L, sizeV[1]-1 do begin
        VxHigh[i] =  V[0,i]*360./dimX
        VyHigh[i] =  FLOAT(header[12])- V[1,i]*ABS(header[12]-header[10])/dimY
        VzHigh[i] =  theta[0] + V[2,i]*(theta[nInFiles-1]-theta[0])/nInFiles
      endfor
      
    endif else begin
      isVertexHighPresent = 0
    endelse
    
  endelse
  
  ;  Saving of all the data in a structure if surface of the volume
  ; with the chosen value is found
  if error eq 0 then begin
  
    volume = { laitPV   : laitPV   $
      , lat      : lat      $
      , lon      : lon      $
      , VxLow    : VxLow    $
      , VyLow    : VyLow    $
      , VzLow    : VzLow    $
      , Plow     : Plow     $
      , VxHigh   : VxHigh   $
      , VyHigh   : VyHigh   $
      , VzHigh   : VzHigh   $
      , PHigh    : PHigh    $
      , theta    : theta    $
      , strYear  : strYear  $
      , strMonth : strMonth $
      , strDay   : strDay   $
      , strHour  : strHour  $
      , isVertexLowPresent  : isVertexLowPresent  $
      , isVertexHighPresent : isVertexHighPresent $
      }
      
  ; Else the function returns 0
  endif else begin
  
    volume = 0
    
  endelse
  
end


;+
; :Description:
;    This procedure is made to plot the isosurface
;
; :Params:
;    inDir, In, Require, Type=string
;         The directory where MIMOSA output PV files are stored
;
;    outDir, In, Require, Type=string
;          The directory where the PNG 24 bit image will be store
;
;    begYear, In, Require, Type=integer
;           The year of the first date to search
;
;    begMonth, In, Require, Type=integer
;            The month of the first date to search
;
;    begDay, In, Require, Type=integer
;           The day of the first date to search
;
;    begHour, In, Require, Type=integer
;           The hour of the first date to search
;
;    endYear, In, Require, Type=integer
;           The year of the last date to search
;
;    endMonth, In, Require, Type=integer
;            The month of the last date to search
;
;    endDay, In, Require, Type=integer
;          The day of the last date to search
;
;    endHour, In, Require, Type=integer
;           The hour of the lat date to search
;
;    stepHour, In, Require, Type=integer
;            The time beteween 2 MIMOSA output files in hours
;
;    normalizeValue, In Require, Type=float
;                  The value of theta0 needed to normalize potential vorticity laitPV = PV*(theta/theta0)^(-9./2.)
;
;    contourValue, In, Require, Type=float
;                The value of the isosurface to plot
;
;    hemisphere, In, Require, Type=string
;              this parameter can take 2 values:
;               - 'N' for an azimuthal projection centered on the north pole
;               - 'S' for an azimuthal projection centered on the north pole
;
; :Keywords:
;    LAT_MIN, In, Optional, Type=float
;           for a plot on the northern hemisphere is the lowest latitude for which the isosurface will be search
;           for a plot on the southern hemisphere is the highest latitude for which the isosurface will be search
;           if not selected lat_min = 60N for hemisphere = 'N' and lat_min = -60N for hemisphere = 'S'
;
;    RESOLUTION, In, Optional, Type=intarr(2)
;              the resolution of the PNG files
;              if not selected PNG images will have a resolution of [2048, 1536]
;              If you chose a lower resolution, It could create problem with comments on the image
;
; :File_comments:
;   If the values of NORMALIZEVALUE and CONTOURVALUE doesn't allow ITTVIS procedure SHADE_VOLUME to find an isosurface in the data
;   it can induce a crash of IDL and IDL workbench
;
;   This program requires the COYOTE library. It is available at http://www.idlcoyote.com/documents/programs.php#COYOTE_LIBRARY_DOWNLOAD
;   Copyright (c) 2011, by Fanning Software Consulting, Inc. All rights reserved.
;
; :Example:
;   To plot all the surface of volume of all MIMOSA output contained in a RUN00 MIMOSA directory:
;     - between the 1st of january 2011 and the 30th of April evey 12 hours
;     - for laitPV normalized with 380K
;     - the surface of the volume with a value of 15 pvu
;     - on the northern hemisphere
;     - for latitude between 50N and 90N
;
;   IDL> MIMOSA_3D_VORTEX, '/home/madrouin/MIMOSA/RUN00/', '/home/madrouin/MIMOSA/PLOT/' $
;          , 2011,  1,  1,  0 $
;          , 2011,  4, 30, 12 $
;          , 12 $
;          , 380., 15. $
;          , 'N' $
;          , LAT_MIN = 50.
;
; :History:
;   201104 MAD/LPC2E Creation
;   201109 MAD/LPC2E add the possibility to plot volume on the south hemisphere
;                    change comments on plots
;                    use of different color palette for internal and external isosurface
;                    add a checking of the isosurface to plot
;                    use of coyote library to manage color
;                    theta in title and axis now appears as a real greek letter
;                    add comments
;
; :Author: madrouin
;-
pro MIMOSA_3D_VORTEX    $
    , inDir             $
    , outDir            $
    , begYear           $
    , begMonth          $
    , begDay            $
    , begHour           $
    , endYear           $
    , endMonth          $
    , endDay            $
    , endHour           $
    , stepHour          $
    , normalizeValue    $
    , contourValue      $
    , hemisphere        $
    , LAT_MIN = lat_min $
    , RESOLUTION = resolution
    
  ; Check path
  lastInDir = STRMID(inDir, STRLEN(inDir)-1, 1)
  if ( STRCMP(lastInDir,'/') eq 0 or STRCMP(lastInDir,'\') eq 0) then inDir = inDir+PATH_SEP()
  lastOutDir = STRMID(outDir, STRLEN(outDir)-1, 1)
  if ( STRCMP(lastOutDir,'/') eq 0 or STRCMP(lastOutDir,'\') eq 0) then outDir = outDir+PATH_SEP()
  
  ; Check KEYWORDS
  if N_ELEMENTS(resolution) eq 0 then resolution = [2048, 1536]
  
  ; Converting gregorian date to julian date and the time step to days
  begJulDay  = JULDAY(begMonth,begDay,begYear,begHour)
  endJulDay  = JULDAY(endMonth,endDay,endYear,endHour)
  stepJulDay = FLOAT(stepHour)/24.
  
  count = 0L
  while (begJulDay le endJulDay) do begin
  
    CALDAT, begJulDay,curMonth,curDay,curYear,curHour
    
    MIMOSA_3D_laitPV_PREPARE $
      , inDir          $
      , curMonth       $
      , curDay         $
      , curHour        $
      , curYear        $
      , normalizeValue $
      , contourValue   $
      , hemisphere     $
      , volume         $
      , LAT_MIN = lat_min
      
    ; Using the Z buffer with 24 bit colors
    ; for 3D plot plot to work is necessary to use /Z_BUFFER keyword
    SET_PLOT, 'Z'
    DEVICE, /Z_BUFFER, SET_RESOLUTION = resolution
    DEVICE, DECOMPOSED = 0
    
    ; Choice of the color palette and color of the differents elements
    cgLOADCT, 1, /SILENT
    blackColor = cgColor('black', !D.Table_Size-2)
    whiteColor = cgColor('White', !D.Table_Size-3)
    greenColor = cgColor('green', !D.TABLE_SIZE-4)
    redColor   = cgColor('red',   !D.TABLE_SIZE-5)
    !P.BACKGROUND = whiteColor
    !P.COLOR = blackColor
    !P.FONT  = 1
    
    ; make sure the background is white
    cgERASE, 'WHITE'
    
    ;  Projection of the maps and field of view
    ; based on chosen hemisphere to plot
    case (hemisphere) of
      'N': begin
        scale3, AX = 45, AZ = 15 $
          , XRANGE = [-180, 180], YRANGE = [0., 90.], ZRANGE = [0, volume.theta[N_ELEMENTS(volume.theta)-1]]
        Map_Set, 90, 0, /AZIMUTHAL, /T3D, /ISOTROPIC, LIMIT=[0, -180, 90, 180], POSITION = [0.05, 0.05, 0.95, 0.95]
      end
      'S': begin
        scale3, AX = 45, AZ = 15 $
          , XRANGE = [-180, 180], YRANGE = [-90., 0.], ZRANGE = [0, volume.theta[N_ELEMENTS(volume.theta)-1]]
        Map_Set, -90, 0, /AZIMUTHAL, /T3D, /ISOTROPIC, LIMIT=[-90, -180, 0, 180], POSITION = [0.05, 0.05, 0.95, 0.95]
      end
      else: begin
        error = 1
        PRINT, 'ERROR : hemisphere should have the value N or S'
      end
    endcase
    
    ; Setting axis
    AXIS, ZAXIS = 2, ZRANGE = [0,volume.theta[N_ELEMENTS(volume.theta)-1]], ZTITLE = Greek('theta')+' (K)', CHARSIZE = 5., /T3D
    
    ; Plot continents fill in black and a grid
    MAP_CONTINENTS, /CONTINENTS, /T3D, /FILL_CONTINENTS, COLOR=blackColor
    MAP_GRID, /T3D
    
    ; Plot of the internal and external surfaces of the volume
    ; check if vertex were found for internal surface
    if volume.isVertexlowPresent EQ 1 then begin
      image = POLYSHADE(volume.VxLow,  volume.VyLow,  volume.VzLow,  volume.PLow,  TOP = 249, /T3D)
    endif else begin
      print, ''
      print, 'Warning no internal surface found for file of the : '+volume.strMonth+'/'+volume.strDay+'/'+volume.strYear+' '+volume.strHour+'H'
    endelse
    ; check if vertex were found for external surface
    if volume.isVertexHighPresent EQ 1 then begin
      cgLOADCT, 3, /SILENT
      blackColor = cgColor('black', !D.Table_Size-2)
      whiteColor = cgColor('White', !D.Table_Size-3)
      greenColor = cgColor('green', !D.TABLE_SIZE-4)
      redColor   = cgColor('red',   !D.TABLE_SIZE-5)
      !P.BACKGROUND = whiteColor
      !P.COLOR = blackColor
      image = POLYSHADE(volume.VxHigh, volume.VyHigh, volume.VzHigh, volume.PHigh, TOP = 249, /T3D)
    endif else begin
      print, ''
      print, 'Warning no external surface found for file of the : '+volume.strMonth+'/'+volume.strDay+'/'+volume.strYear+' '+volume.strHour+'H'
    endelse
    
    ; Add contour projection on the map for theta = 450K and max(theta)
    maxThetaVertex = Max(volume.VzHigh)
    tmp = MIN(ABS(maxThetaVertex-volume.theta), maxThetaIndex)
    contour, volume.laitPV[*,*,maxThetaIndex], volume.lon, volume.lat $
      , levels = [contourValue], ZVALUE=0., COLOR = redColor $
      , C_THICK=[4.] $
      , /T3D, /OVERPLOT
    minThetaVertex = Max(volume.VzHigh)
    tmp = MIN(ABS(450.-volume.theta), minThetaIndex)
    contour, volume.laitPV[*,*,minThetaIndex+1], volume.lon, volume.lat $
      , levels = [contourValue], ZVALUE=0., COLOR = greenColor $
      , C_THICK=[4.] $
      , /T3D, /OVERPLOT
      
    ; Add title and comments on the plot
    XYOUTS, 0.95, 0.53, 'Surface: '+ STRING(contourValue, FORMAT='(I2)')+' pvu' $
      , ALIGNMENT=1., CHARTHICK=3., CHARSIZE=3., /NORMAL, COLOR=blackColor
    XYOUTS, 0.95, 0.5 $
      , 'Contours on map' $
      , ALIGNMENT=1., CHARTHICK=3., CHARSIZE=3., /NORMAL, COLOR=blackColor
    XYOUTS, 0.95, 0.47 $
      , greek('theta')+' = '+STRING(450,FORMAT='(I4)')+'K', COLOR = greenColor $
      , ALIGNMENT=1., CHARTHICK=3., CHARSIZE=3., /NORMAL
    XYOUTS, 0.95, 0.44 $
      , greek('theta')+' = '+STRING(FIX(volume.theta[maxThetaIndex]),FORMAT='(I4)')+'K', color = redColor $
      , ALIGNMENT=1., CHARTHICK=2., CHARSIZE=3., /NORMAL
      
    XYOUTS, 0.5, 0.95 $
      , 'MIMOSA : lait PV normalized with '+Greek('theta')+'!I0!N='+STRING(normalizeValue,FORMAT='(I3)')+'K' $
      , ALIGNMENT=0.5, CHARTHICK=2., CHARSIZE=3., /NORMAL
    XYOUTS, 0.5, 0.90 $
      , volume.strMonth+'/'+volume.strDay+'/'+volume.strYear+' '+volume.strHour+'H' $
      , ALIGNMENT=0.5, CHARTHICK=2., CHARSIZE=3., /NORMAL
      
    strContourValue = STRING(contourValue, FORMAT='(I2.2)')
    
    ; Name of PNG image
    fileOut = outDir $
      + '/mimosa_3D_' + strContourValue + '_' $
      + volume.strYear + volume.strMonth + volume.strDay + volume.strHour + '.png'
      
    ; Saved PNG image
    WRITE_PNG, fileOut, TVRD(/TRUE)
    
    ; Next step
    begJulDay = begJulDay+stepJulDay
    count++
    
  endwhile
  
  PRINT, ''
  PRINT, 'End of plot'
  
end

MIMOSA_3D_VORTEX, '/data/ENRICHED/MIMOSA/VIDEO_VORTEX/input/' $
  , '/home/madrouin/test/IDL_Plot3D/' $
  , 2011,1,15,12,2011,4,30,12,12,380,15, 'N' $
  , LAT_MIN=40.
  
end
