;+
; :Description:
;    The purpose of this routine is to plot the temperature fields outputted by MIMOSA model.
;    A contour plot on the projection of a map will be plotted.
;    The program outputs 24bit PNG image.
;
;    Warning: when the levels of the contours are calculated by the procedure,
;    only the first file to plot is used so it's possible the levels don't suit
;    the range value of all files.
;
; :Categories:
;    MIMOSA tools
;
; :Params:
;    filelist, In, Require, Type=string
;             The Path+name of the ASCII file containing the list of file to plot
;
;    inputDir, In, Require, Type=string
;             The path of the directory where MIMOSA output files are stored
;
; :Keywords:
;    OUTPUTDIR, In, Optional, Type=string
;              If selected the PNG images will be save in this directory.
;              If not PNG images will be saved in inputDir
;
;    LEVELS, In, Optional, Type=fltarr
;           The list of levels of the filled contours.
;           If not selected, the program will determine a list of 20 contours based on
;          the MIN and MAX of the PV grid
;
;    NO_FILL_LEVELS, In, Optional, Type=fltarr
;                   The list of levels of white contour on the map.
;                   If not selected, the program will used the same levels as the one given by LEVELS keyword.
;
;    CCOLORS, In, Optional, Type=intarr
;            The list of colors you want to use to fill contour.
;            If not selected, the colors will be determine based on the number of values of LEVELS.
;
;    NPALETTE, In, Optional, Type=int
;             The color palette you want to use for the plot.
;             if not selected palette number 5 is used.
;             Whatever the palette used the background of the map will always be white and the color of lines black
;
;    LAT_RANGE, In, Optional, Type=fltarr
;              the range of latitude to plot. If not selected the plot will be done between 0N and 90N or -90N and 0N
;             based on the chosen map projection
;
;    LON_RANGE, In, Optional, Type=fltarr
;              the range of longitude to plot. if not selected the plot will be done between -180E and 180E
;             these values should always be between -180. and 180
;
;    SOUTH_AZIMUTHAL, In, Optional, Type=logical
;                    If selected the map projection wil be azimuthal centered on the south pole.
;                    If not selected an azimuthal map projection centered on the north pole will be use.
;
;    CYLINDRICAL, In, Optional, Type=logical
;                If selected the map projection wil be azimuthal centered on the south pole.
;                If not selected an azimuthal map projection centered on the north pole will be use.
;
; :Examples:
;   To plot
;   IDL> fileList = '/data/STRAPOLETE/POST_CAMPAIGN/T_profiles/2000/data/list_file_2000.in'
;   IDL> inpuDir = '/data/STRAPOLETE/POST_CAMPAIGN/T_profiles/2000/data/'
;   IDL> MIMOSA_PLOT_PV, fileList, inputDir
;
; :File_comments:
;   This function needs STRMONTH, MIMOSA_READ_LIST, MIMOSA_READ_BIN and colorbar to work
;
;   to create easily a list file on LINUX
;   to go the directory where the file are located in a terminal
;   $ cd /data/STRAPOLETE/POST_CAMPAIGN/T_profiles/2000/data
;   $ ls t* | wc -l > list_file_2000.in
;   $ ls t*  >> list_file_2000.in
;   The file is ready to be read by the MIMOSA_PLOT_T procedure
;
;   This program requires the COYOTE library. It is available at http://www.idlcoyote.com/documents/programs.php#COYOTE_LIBRARY_DOWNLOAD
;   Copyright (c) 2011, by Fanning Software Consulting, Inc. All rights reserved.
;
; :
;
; :History:
;   200903 MAD/LPC2E Creation
;   201005 MAD/LPC2E Add keywords and comments
;   201109 MAD/LPC2E Add keywords NO_FILL_LEVELS, LAT_RANGE, LON_RANGE, SOUTH_AZIMUTHAL and CYLINDRICAL
;                    Change comments to use IDLdoc format
;
;
; :Author: madrouin
;-
PRO MIMOSA_PLOT_T                        $
    , filelist                           $
    , inputDir                           $
    , OUTPUTDIR        = outputDir       $
    , LEVELS           = levels          $
    , NO_FILL_LEVELS   = nfLevels        $
    , CCOLORS          = ccolors         $
    , NPALETTE         = npalette        $
    , LAT_RANGE        = latRange        $
    , LON_RANGE        = lonRange        $
    , SOUTH_AZIMUTHAL  = south_azimuthal $
    , CYLINDRICAL      = cylindrical
    
  ; Check inputDir format
  lastInDir = STRMID(inputDir, STRLEN(inputDir)-1, 1)
  if ( STRCMP(lastInDir,'/') eq 0 or STRCMP(lastInDir,'\') eq 0) then inputDir = inputDir+PATH_SEP()
  
  ; Check outputDir format
  if ~KEYWORD_SET(OUTPUTDIR) then outputDir = inputDir
  lastOutDir = STRMID(outputDir, STRLEN(outputDir)-1, 1)
  if ( STRCMP(lastOutDir,'/') eq 0 or STRCMP(lastOutDir,'\') eq 0) then outputDir = outputDir+PATH_SEP()
  
  ; Read the list of mimosa potential vorticity files to process
  nameFile = mimosa_read_list(fileList)
  nfile    = N_ELEMENTS(nameFile)
  
  PRINT, ''
  PRINT, 'MIMOSA: Plot temperature'
  PRINT, STRING(nfile, FORMAT='(I4)'), ' files to plot'
  
  ; Graphical options
  if KEYWORD_SET(LEVELS) then begin
    nlevels = N_ELEMENTS(levels)
  endif else begin
    nlevels = 20
  endelse
  
  if KEYWORD_SET(CCOLORS) then begin
    ncolors = N_ELEMENTS(ccolors)
    if (ncolors ne nlevels-1) then begin
      PRINT, ''
      PRINT, 'WARNING: incorrect number of color'
      PRINT, 'Number of colors should be number of levels-1'
      PRINT, ''
    endif
  endif else begin
    ncolors = nlevels-1
  endelse
  
  if N_ELEMENTS(NPALETTE) eq 0 then begin
    npalette = 5
  endif else begin
    if (npalette GT 40) then begin
      PRINT, ''
      PRINT, 'ERROR: palette ', STRING(npalette, FORMAT='(I3)'), ' doesn t exist'
      stop
    endif
  endelse
  
  if (N_ELEMENTS(size(LONRANGE)) lt 4) then lonRange = [-180, 180.]
  
  if ~KEYWORD_SET(SOUTH_AZIMUTHAL) then begin
    if (N_ELEMENTS(size(LATRANGE)) lt 4) then latRange = [0., 90.]
  endif else begin
    if (N_ELEMENTS(size(LATRANGE)) lt 4) then latRange = [-90., 0.]
  endelse
  
  ; Processing files to plot
  FOR i = 0, nFile-1 DO BEGIN
  
    MIMOSA_READ_BIN, inputDir + nameFile[i], HEADER=header, DATA = t
    
    hem   = STRMID(nameFile[i], 1, 1)
    ; Getting caracteristics contains in header
    year  = header[0]
    month = header[1]
    day   = header[2]
    hour  = header[3]
    theta = header[8]
    
    ; Convert date to string
    strYear  = STRING(year, FORMAT='(I2.2)')
    strMonth = STRING(month, FORMAT='(I2.2)')
    strDay   = STRING(day, FORMAT='(I2.2)')
    strHour  = STRING(hour, FORMAT='(I2.2)')
    strTheta = STRING(theta, FORMAT='(I4.4)')
    
    ; Get resolution of the grid
    dimX = header[19] & dimY = header[20]
    
    ; define longitude and latitude vector
    long = FINDGEN(dimX)*360./(dimX-1)
    lat  = header[12] - FINDGEN(dimY)*(ABS(header[12]-header[10]))/dimY
    
    ;# Parametres graphiques
    if (i eq 0) then begin
      SET_PLOT , 'Z'
      DEVICE, DECOMPOSED = 0, Z_BUFFERING = 0, SET_RESOLUTION = [2048, 1536]
      cgLOADCT, npalette, /SILENT
      blackColor = cgColor('black', !D.Table_Size-2)
      whiteColor = cgColor('white', !D.Table_Size-3)
      !P.BACKGROUND = whiteColor
      !P.COLOR = blackColor
      !P.FONT  = 1
      
      ; Auto calculating levels if needed
      if ~KEYWORD_SET(LEVELS)  then levels  = MIN(t) + FINDGEN(nlevels)*ABS((MAX(DOUBLE(t))-MIN(DOUBLE(t))))/DOUBLE(nlevels-1)
      if ~KEYWORD_SET(nfLevels) then nfLevels = levels
      
      ; Auto calculating colors if needed
      if ~KEYWORD_SET(CCOLORS) then ccolors = 1+INDGEN(ncolors)*(!D.Table_Size-3)/(ncolors)
      tmpColors = ccolors      
    endif
    
    ; Get name of the month
    month_string = STR_MONTH(month)
    if (year LT 51) then begin
      year_string = '20' + strYear
    endif else begin
      year_string = '19' + strYear
    endelse
    
    Titre = 'MIMOSA: Temperature (K) ' $
      + month_string + ' ' + strDay + ' ' + year_string + ' ' + strHour +'UT'
      
    cgErase, 'white'
    ccolors = tmpColors
    
    if KEYWORD_SET(CYLINDRICAL) then begin
      MAP_SET, /CYLINDRICAL, LIMIT = [latRange[0], lonRange[0], latRange[1], lonRange[1]], POSITION = [0.05,0.15,0.95,0.925], /NOBORDER
    endif else if KEYWORD_SET(SOUTH_AZIMUTHAL) then begin
      MAP_SET, -90, 0, /AZIMUTHAL, /ISOTROPIC, LIMIT = [latRange[0], lonRange[0], latRange[1], lonRange[1]], POSITION = [0.155,0.,0.845,0.925]
    endif else begin
      MAP_SET, 90, 0, /AZIMUTHAL, /ISOTROPIC, LIMIT = [latRange[0], lonRange[0], latRange[1], lonRange[1]], POSITION = [0.155,0.,0.845,0.925]
    endelse
    
    
    CONTOUR, t, Long, Lat, /CELL_FILL, LEVELS = levels $
      , C_COLOR = CColors, /OVERPLOT
      
    CONTOUR, t, Long, Lat, LEVELS = nfLevels $
      , C_LABEL=nfLevels, C_CHARSIZE = 2.0, COLOR=whiteColor $
      , /FOLLOW, /OVERPLOT
      
    if ~KEYWORD_SET(CYLINDRICAL) then begin
    
      MAP_CONTINENTS, /COUNTRIES, /COASTS
      
      MAP_GRID, LATS = -90+INDGEN(19)*10
      
      COLORBAR, /VERTICAL, /RIGHT, POSITION = [ 0.855, 0.01, 0.890, 0.925], FORMAT = '(F7.1)' $
        , DISCRETE = CColors $
        , CB_LABEL = Levels, CB_CHARSIZE = 3., CB_CHARTHICK = 2., CB_COLOR = blackColor
        
      XYOUTS, 0.01, 0.90, strTheta + ' K', CHARSIZE = 4., /NORMAL, ALIGNMENT=0.0, COLOR=blackColor
      XYOUTS, 0.50, 0.95, Titre, CHARSIZE = 4., /NORMAL, ALIGNMENT=0.5, COLOR=blackColor
      
      ; Name of output file
      fileOut = outputDir + 't' + strYear + strMonth + strDay + strHour $
        + '_' + hem + strTheta + '.png'
        
    endif else begin
    
      MAP_CONTINENTS, /COUNTRIES, /COASTS, MLINETHICK=2.
      
      MAP_GRID, /BOX_AXES, CHARSIZE=3.
      
      COLORBAR, POSITION = [ 0.05, 0.05, 0.95, 0.1], FORMAT = '(F7.1)' $
        , DISCRETE = CColors $
        , CB_LABEL = Levels, CB_CHARSIZE = 2., CB_CHARTHICK = 2., CB_COLOR = blackColor
        
      XYOUTS, 0.01, 0.96, strTheta + ' K', CHARSIZE = 4., /NORMAL, ALIGNMENT=0.0, COLOR=blackColor
      XYOUTS, 0.50, 0.96, Titre, CHARSIZE = 4., /NORMAL, ALIGNMENT=0.5, COLOR=blackColor
      
      ; Name of output file
      fileOut = outputDir + 't_CYL' + strYear + strMonth + strDay + strHour $
        + '_' + hem + strTheta + '.png'
        
    endelse
    
    WRITE_PNG, FileOut, TVRD(/TRUE)
    
  ENDFOR
  
  DEVICE, /CLOSE
  
  PRINT, ''
  PRINT, 'End'
  PRINT, 'Files were saved in ', OUTPUTDIR
  
END